%Module 3 Lesson 3- Series Mode Optimization
clear
clc
close all

%Load generator .mat file
load('generator.mat');

%Plot generator efficiency map
N_gen=generator.N_RPM;      %Generator speed     (RPM)
T_gen=generator.T_Nm;       %Generator torque    (Nm)
eff_gen=generator.Eff_pct;  %Motor efficiency
N_gen_maxmin=generator.max_min_w*30/pi;     %Speeds corresponding to max/min generator torques (RPM)
T_gen_max=generator.T_max_Peak;             %Maximum generator torques 
T_gen_min=generator.T_min_Peak;             %Minimum generator torques 

[N_gen_grid,T_gen_grid]=meshgrid(N_gen,T_gen); %Create mesh to be used in contour plot

figure
[C,h]= contour(N_gen_grid,T_gen_grid,eff_gen,[.1 .5 .70 .80 .90 .93 .94]);
clabel(C,h);
hold on
plot(N_gen_maxmin,T_gen_max,'k',N_gen_maxmin,T_gen_min,'k','linewidth',4)
xlim([min(N_gen_maxmin),max(N_gen_maxmin)])
ylim([min(T_gen_min),max(T_gen_max)])
xlabel('Motor Speed [RPM]','Fontsize',12,'fontweight','bold')
ylabel('Motor Torque [Nm]','Fontsize',12,'fontweight','bold')
title('Motor Efficiency','Fontsize',12,'fontweight','bold')
hold off

%Load engine .mat file
load('engine.mat');

%Plot engine efficiency map
N_eng_pos=[1200:30:4200];      %Range of possible engine speeds
T_eng_pos=[0:5:150 159.15];    %Range of possible engine torques

[N_eng_grid,T_eng_grid]=meshgrid(N_eng_pos,T_eng_pos);     %Create mesh to be used for contour
[N_grid,T_grid,Eng_eff]=griddata(N_eng,T_eng_Nm,Eff_eng,N_eng_grid,T_eng_grid);    %Relate efficiency to speed/torque
 
figure
hold on
[C,h]= contour(N_grid,T_grid,Eng_eff,[5:5:35 37 39 40 41]);
clabel(C,h)
plot(N_eng_maxT,T_eng_maxNm,'k','linewidth',4)
xlabel('Engine Speed [RPM]','Fontsize',16,'fontweight','bold')
ylabel('Engine Brake Torque [Nm]','Fontsize',16,'fontweight','bold')
title('Engine Efficiency [%]','Fontsize',16,'fontweight','bold')

%Optimize for best engine/generator operating points
%%%%%%%%%%%%% FIXED GEAR RATIO %%%%%%%%%%%%%%%%%%%%%%%%%%%
gear=1.75;   %Gear ratio (Engine:Generator)

Motor_eff=interp2(N_gen_grid,T_gen_grid,eff_gen,N_grid*gear,T_grid./gear); %Find motor efficiencies at same operating points as engine
Eff=Motor_eff.*Eng_eff;    %Combined engine/generator efficiency to be used to find most efficient operating points

P=1:1:60;     %Range of possible generator powers to be used in series mode

for m=1:length(P);   %Calculate engine torque corresponding to each power and engine speed
    for j=1:length(N_eng_pos);
        if (interp1(N_eng_maxT,T_eng_maxNm,N_eng_pos(j)))>P(m)*9549/N_eng_pos(j) ...
         && (interp1(N_gen_maxmin,abs(T_gen_min),N_eng_pos(j)*gear)) > P(m)*9549/N_eng_pos(j)/gear;   %Check to make sure torque is not above generator/engine max
           T(j)=P(m)*9549./N_eng_pos(j);   
        else
           T(j)=0;    %If torque is above max, set to 0
        end
     end
  
        [A]=find(T);                  %Retrieve all non-zero torques
        T_kw{m}=T(A);                 %Create cell of each torque vector corresponding to each power value
        N_eng_kw{m}=N_eng_pos(A);     %Create cell of each speed vector corresponding to each power value

    Eff_interp=interp2(N_grid,T_grid,Eff,N_eng_kw{m},T_kw{m});   %Find efficiency corresponding to each torque/speed operating point
  
    [max_eff(m),k]=max(Eff_interp);   %Find maximum efficiency for each power value
    N_effmax(m)=N_eng_kw{m}(k);       %Find corresponding engine speed
    T_effmax(m)=T_kw{m}(k);           %Find corresponding engine torque
end

figure   %Plot combined efficiency map with each power line and corresponding operating point
hold on
[C,h]= contour(N_grid,T_grid,Eff,[5:5:30 32 34:38]);
clabel(C,h)
for i=1:length(P)
    plot(N_eng_kw{i},T_kw{i},'--k','linewidth',2)
end
plot(N_effmax,T_effmax,'-*','linewidth',2)
plot(N_eng_maxT,T_eng_maxNm,'k','linewidth',4)
xlabel('Engine Speed [RPM]','Fontsize',16,'fontweight','bold')
ylabel('Brake Torque [Nm]','Fontsize',16,'fontweight','bold')
title('Efficiency [%] at GR=1.75 for Powers 1-60 kW','Fontsize',16,'fontweight','bold')

Eff_elec=interp2(N_gen_grid,T_gen_grid,eff_gen,N_effmax.*gear,T_effmax./gear);   %Find generator efficiency corresponding to each operating point
P_elec=N_effmax.*T_effmax.*Eff_elec/9549;   %Using generator efficiency, find actual electric power produced at each point
                                      %Will be used in SOC control later on                                            %lat


%% If you were to operate at one fixed point (thermostatic control), find power range and find most efficient point
%Can also optimize gear ratio

P2=30:1:60;

gear=1.0:.25:1.75;

for l=1:length(P2);
    for i=1:length(gear);
            for j=1:length(N_eng_pos);
                if (interp1(N_eng_maxT,T_eng_maxNm,N_eng_pos(j)))>P2(l)*9549/N_eng_pos(j) ...
                     && (interp1(N_gen_maxmin,abs(T_gen_min),N_eng_pos(j)*gear(i))) > P2(l)*9549/N_eng_pos(j)/gear(i);
                    T2(j)=P2(l)*9549./N_eng_pos(j);
                else
                    T2(j)=0;
                end
            end
  
        [A]=find(T2);
        T_kw2=T2(A);
        N_eng_kw2=N_eng_pos(A);
    
        Motor_eff=interp2(N_gen_grid,T_gen_grid,eff_gen,N_grid*gear(i),T_grid./gear(i));

        Eff=Motor_eff.*Eng_eff;

%         figure
%         hold on
%         [C,h]= contour(N_grid,T_grid,Eff,[5:5:30 32 34:38]);
%         clabel(C,h)
%         plot(N_eng_kw,T_kw,'--k','linewidth',4)
%         plot(N_eng_maxT,T_eng_maxNm,'k','linewidth',4)
%         xlabel('Engine Speed [RPM]','Fontsize',16,'fontweight','bold')
%         ylabel('Brake Torque [Nm]','Fontsize',16,'fontweight','bold')
%         title('Brake Efficiency [%]','Fontsize',16,'fontweight','bold')
        Eff_30kw=interp2(N_grid,T_grid,Eff,N_eng_kw2,T_kw2);

    
        [max_eff_30kw(i),k]=max(Eff_30kw);
        N_effmax_30kw(i)=N_eng_pos(k);
        T_effmax_30kw(i)=T_kw2(k);
    end

    [eff_max(l),k]=max(max_eff_30kw);
    x=find(max_eff_30kw==eff_max(l));
    gear_final(l)=gear(x);
    N_effmax(l)=N_eng_kw2(k);
    T_effmax(l)=T_kw2(k);    
end

[Eff_best,D]=max(eff_max);
Eff_best;
N_best=N_effmax(D);
T_best=T_effmax(D);
gear_best=gear_final(D);
Power=P2(D);

Motor_eff=interp2(N_gen_grid,T_gen_grid,eff_gen,N_grid*gear_best,T_grid./gear_best);

Eff=Motor_eff.*Eng_eff;

            for j=1:length(N_eng_pos);
                if (interp1(N_eng_maxT,T_eng_maxNm,N_eng_pos(j)))>P2(D)*9549/N_eng_pos(j) ...
                     && (interp1(N_gen_maxmin,abs(T_gen_min),N_eng_pos(j)*gear_best)) > P2(D)*9549/N_eng_pos(j)/gear_best;
                    T_kw2(j)=P2(D)*9549./N_eng_pos(j);
                else
                    T_kw2(j)=0;
                end
            end
  
        [A]=find(T_kw2);
        T_kw2=T_kw2(A);
        N_eng_kw2=N_eng_pos(A);
        
figure
hold on
[C,h]= contour(N_grid,T_grid,Eff,[5:5:30 32 34:38]);
clabel(C,h)
plot(N_eng_kw2,T_kw2,'--k','linewidth',2)
plot(N_eng_maxT,T_eng_maxNm,'k','linewidth',4)
plot(N_best,T_best,'b*','linewidth',4)
str=sprintf('Brake Efficiency at GR= %.2f at %.0f kW',gear_best,Power);
xlabel('Engine Speed [RPM]','Fontsize',16,'fontweight','bold')
ylabel('Brake Torque [Nm]','Fontsize',16,'fontweight','bold')
title(str,'Fontsize',16,'fontweight','bold')